﻿// Copyright 2009-2014 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using FCSlib.Data.Collections;
using FCSlib;
using FCSlib.Data;

namespace Algorithms {
  class Program {
    static void Main(string[] args) {
      var list = new List<int>(1, 2, 3, 4, 5, 6, 7, 8, 9, 10);
      
      // Both of these are recursive, so may not be usable or 
      // efficient in C#. SumCPS would be fine in many 
      // languages though (and it might look more elegant, too).
      Console.WriteLine(SumRec(list));
      SumCPS(list, x => Console.WriteLine(x));

      // This implementation uses a standard higher order function
      // That function might well use mutable values, but they
      // are not visible for this code, so they are somebody else's problem (tm).
      Console.WriteLine(Sum(list));
    }



    static int SumRec(List<int> list) {
      return list.IsEmpty ? 0 : list.Head + SumRec(list.Tail);
    }

    static void SumCPS(List<int> list, Action<int> continuation) {
      if (list.IsEmpty)
        continuation(0);
      else
        SumCPS(list.Tail, x => continuation(x + list.Head));
    }

    static int Sum(List<int> list) {
      return Functional.FoldL((r, v) => r + v, 0, list);
    }

  }
}
