// Copyright 2009-2010 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Linq.Expressions;

namespace FCSlib.Data {
  public static class CloningExtensions {
    public static T CloneWith<T>(this T source, Dictionary<string, object> newValues) {
      var typet = typeof(T);
      var ctors = typet.GetConstructors( );
      if (ctors.Length > 1)
        throw new InvalidOperationException(String.Format("Can't clone type {0} because it has more than one constructor.", typet));
      var ctor = ctors[0];
      var cparams = ctor.GetParameters();
      var paramCount = cparams.Length;
      var paramArray = new object[paramCount];

      for (int i = 0; i < paramCount; i++) {
        if (newValues.ContainsKey(cparams[i].Name))
          paramArray[i] = newValues[cparams[i].Name];
        else
          paramArray[i] = GetAccessor(typet, cparams[i].Name)(source);
      }

      return (T) ctor.Invoke(paramArray);
    }

    static Dictionary<Type, Dictionary<string, Func<object, object>>> accessors =
      new Dictionary<Type, Dictionary<string, Func<object, object>>>( );

    static Func<object, object> GetAccessor(Type type, string valueName) {
      Func<object, object> result;
      Dictionary<string, Func<object, object>> typeAccessors;

      // Let's see if we have an accessor already for this type/valueName
      if (accessors.TryGetValue(type, out typeAccessors)) {
        if (typeAccessors.TryGetValue(valueName, out result))
          return result;
      }

      // okay, create one and store it for later
      result = CreateAccessor(type, valueName);
      if (typeAccessors == null) {
        typeAccessors = new Dictionary<string, Func<object, object>>( );
        accessors[type] = typeAccessors;
      }
      typeAccessors[valueName] = result;

      return result;
    }

    static Func<object, object> CreateAccessor(Type type, string valueName) {
      var finfo = type.GetField(valueName);
      var param = Expression.Parameter(typeof(object), "o");
      Expression<Func<object, object>> exp =
        Expression.Lambda<Func<object, object>>(
        Expression.Convert(
        Expression.Field(Expression.Convert(param, type), finfo), typeof(object)),
        param);
      return exp.Compile( );
    }
  }
}
