// Copyright 2009-2010 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace FCSlib.Data {
  public class Tuple<T1, T2> {
    private T1 val1;
    public T1 Val1 {
      get { return val1; }
    }
    private T2 val2;
    public T2 Val2 {
      get { return val2; }
    }

    public Tuple(T1 val1, T2 val2) {
      this.val1 = val1;
      this.val2 = val2;
    }

    public override string ToString( ) {
      return String.Format("T({0}, {1})", val1, val2);
    }

    public static bool operator ==(Tuple<T1, T2> a, Tuple<T1, T2> b) {
      return
        EqualityComparer<T1>.Default.Equals(a.Val1, b.Val1) &&
        EqualityComparer<T2>.Default.Equals(a.Val2, b.Val2);
    }

    public static bool operator !=(Tuple<T1, T2> a, Tuple<T1, T2> b) {
      return !(a == b);
    }

    public override int GetHashCode( ) {
      return val1.GetHashCode( ) ^ val2.GetHashCode( );
    }

    public override bool Equals(object obj) {
      return this == (Tuple<T1, T2>) obj;
    }
  }
  public class Tuple<T1, T2, T3> {
    private T1 val1;
    public T1 Val1 {
      get { return val1; }
    }
    private T2 val2;
    public T2 Val2 {
      get { return val2; }
    }
    private T3 val3;
    public T3 Val3 {
      get { return val3; }
    }

    public Tuple(T1 val1, T2 val2, T3 val3) {
      this.val1 = val1;
      this.val2 = val2;
      this.val3 = val3;
    }

    public override string ToString( ) {
      return String.Format("T({0}, {1}, {2})", val1, val2, val3);
    }

    public static bool operator ==(Tuple<T1, T2, T3> a, Tuple<T1, T2, T3> b) {
      return
        EqualityComparer<T1>.Default.Equals(a.Val1, b.Val1) &&
        EqualityComparer<T2>.Default.Equals(a.Val2, b.Val2) &&
        EqualityComparer<T3>.Default.Equals(a.Val3, b.Val3);
    }

    public static bool operator !=(Tuple<T1, T2, T3> a, Tuple<T1, T2, T3> b) {
      return !(a == b);
    }

    public override int GetHashCode( ) {
      return val1.GetHashCode( ) ^ val2.GetHashCode( ) ^ val3.GetHashCode( );
    }

    public override bool Equals(object obj) {
      return this == (Tuple<T1, T2, T3>) obj;
    }

  }

  public class Tuple<T1, T2, T3, T4> {
    private T1 val1;
    public T1 Val1 {
      get { return val1; }
    }
    private T2 val2;
    public T2 Val2 {
      get { return val2; }
    }
    private T3 val3;
    public T3 Val3 {
      get { return val3; }
    }
    private T4 val4;
    public T4 Val4 {
      get { return val4; }
    }
    public Tuple(T1 val1, T2 val2, T3 val3, T4 val4) {
      this.val1 = val1;
      this.val2 = val2;
      this.val3 = val3;
      this.val4 = val4;
    }

    public override string ToString( ) {
      return String.Format("T({0}, {1}, {2}, {3})", val1, val2, val3, val4);
    }

    public static bool operator ==(Tuple<T1, T2, T3, T4> a, Tuple<T1, T2, T3, T4> b) {
      return
        EqualityComparer<T1>.Default.Equals(a.Val1, b.Val1) &&
        EqualityComparer<T2>.Default.Equals(a.Val2, b.Val2) &&
        EqualityComparer<T3>.Default.Equals(a.Val3, b.Val3) &&
        EqualityComparer<T4>.Default.Equals(a.Val4, b.Val4);
    }

    public static bool operator !=(Tuple<T1, T2, T3, T4> a, Tuple<T1, T2, T3, T4> b) {
      return !(a == b);
    }

    public override int GetHashCode( ) {
      return val1.GetHashCode( ) ^ val2.GetHashCode( ) ^ val3.GetHashCode( ) ^ val4.GetHashCode( );
    }

    public override bool Equals(object obj) {
      return this == (Tuple<T1, T2, T3, T4>) obj;
    }

  }
  public class Tuple<T1, T2, T3, T4, T5> {
    private T1 val1;
    public T1 Val1 {
      get { return val1; }
    }
    private T2 val2;
    public T2 Val2 {
      get { return val2; }
    }
    private T3 val3;
    public T3 Val3 {
      get { return val3; }
    }
    private T4 val4;
    public T4 Val4 {
      get { return val4; }
    }
    private T5 val5;
    public T5 Val5 {
      get { return val5; }
    }
    public Tuple(T1 val1, T2 val2, T3 val3, T4 val4, T5 val5) {
      this.val1 = val1;
      this.val2 = val2;
      this.val3 = val3;
      this.val4 = val4;
      this.val5 = val5;
    }

    public override string ToString( ) {
      return String.Format("T({0}, {1}, {2}, {3}, {4})", val1, val2, val3, val4, val5);
    }

    public static bool operator ==(Tuple<T1, T2, T3, T4, T5> a, Tuple<T1, T2, T3, T4, T5> b) {
      return
        EqualityComparer<T1>.Default.Equals(a.Val1, b.Val1) &&
        EqualityComparer<T2>.Default.Equals(a.Val2, b.Val2) &&
        EqualityComparer<T3>.Default.Equals(a.Val3, b.Val3) &&
        EqualityComparer<T4>.Default.Equals(a.Val4, b.Val4) &&
        EqualityComparer<T5>.Default.Equals(a.Val5, b.Val5);
    }

    public static bool operator !=(Tuple<T1, T2, T3, T4, T5> a, Tuple<T1, T2, T3, T4, T5> b) {
      return !(a == b);
    }

    public override int GetHashCode( ) {
      return val1.GetHashCode( ) ^ val2.GetHashCode( ) ^ val3.GetHashCode( ) ^ val4.GetHashCode( ) ^ val5.GetHashCode( );
    }

    public override bool Equals(object obj) {
      return this == (Tuple<T1, T2, T3, T4, T5>) obj;
    }

  }

  public static class Tuple {
    public static Tuple<T1, T2> Create<T1, T2>(T1 val1, T2 val2) {
      return new Tuple<T1, T2>(val1, val2);
    }
    public static Tuple<T1, T2, T3> Create<T1, T2, T3>(T1 val1, T2 val2, T3 val3) {
      return new Tuple<T1, T2, T3>(val1, val2, val3);
    }
    public static Tuple<T1, T2, T3, T4> Create<T1, T2, T3, T4>(T1 val1, T2 val2, T3 val3, T4 val4) {
      return new Tuple<T1, T2, T3, T4>(val1, val2, val3, val4);
    }
    public static Tuple<T1, T2, T3, T4, T5> Create<T1, T2, T3, T4, T5>(T1 val1, T2 val2, T3 val3, T4 val4, T5 val5) {
      return new Tuple<T1, T2, T3, T4, T5>(val1, val2, val3, val4, val5);
    }
  }
}