﻿using System;
using System.Collections.Generic;
using System.Collections.Immutable;
using System.Linq;
using System.Text;
using System.Threading;

namespace MicrosoftImmutableCollections {
  class Program {
    static void Main(string[] args) {
      var rubberChicken = new Product("Rubber chicken", 5.98m);
      var pulley = new Product("Pulley", 2.99m);
      //var products_ = ImmutableList.Create(rubberChicken, pulley);
      products = ImmutableList<Product>.Empty.Add(rubberChicken).Add(pulley);
      var order = new Order(DateTime.Today,
        ImmutableList<OrderLine>.Empty.Add(
          new OrderLine(rubberChicken, 3)).Add(
          new OrderLine(pulley, 3)
        ));

      // run the real world simulator
      new Thread((ParameterizedThreadStart) RealWorldSimulator) {
        IsBackground = true
      }.Start(order);

      // run this test several times 
      for (int i = 0; i < 100; i++)
        TestOrderValue(order);
    }
  
    static ImmutableList<Product> products;

    static void TestOrderValue(Order order) {
      try {
        if (order.GetValue( ) == (3 * 5.98m + 3 * 2.99m))
          Console.WriteLine("Everything's cool");
        else
          Console.WriteLine("Argh!");
      }
      catch (Exception ex) {
        Console.WriteLine("Caught exception: " + ex.Message);
      }
    }

    static void RealWorldSimulator(object orderObj) {
      var order = (Order) orderObj;
      while (true) {
        // do things to the order, in the way preferred by the API
        while (true)
          order = order.AddLine(new OrderLine(products[0], 2));
      }
    }

  }

  public class Product {
    private readonly string name;
    public string Name { get { return name; } }
    private readonly decimal price;
    public decimal Price { get { return price; } }

    public Product(string name, decimal price) {
      this.name = name;
      this.price = price;
    }
  }

  public class OrderLine {
    private readonly Product product;
    public Product Product { get { return product; } }
    private readonly int count;
    public int Count { get { return count; } }

    public OrderLine ChangeCount(int newCount) {
      return new OrderLine(product, newCount);
    }

    public decimal GetValue( ) {
      return Product.Price * Count;
    }

    public OrderLine(Product product, int count) {
      this.product = product;
      this.count = count;
    }
  }

  public class Order {
    private readonly DateTime date;
    public DateTime Date { get { return date; } }
    private readonly ImmutableList<OrderLine> lines;
    public ImmutableList<OrderLine> Lines { get { return lines; } }

    public decimal GetValue( ) {
      decimal result = 0;
      foreach (var line in lines)
        result += line.GetValue( );
      return result;
    }

    public Order AddLine(OrderLine line) {
      return new Order(date, lines.Add(line));
    }

    public Order(DateTime date, ImmutableList<OrderLine> lines) {
      this.date = date;
      this.lines = lines;
    }
  }
}
