﻿// Copyright 2009-2014 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace ImmutableData {
  class Program {
    static void Main(string[] args) {
      var product = new Product("Rubber boat", 16.99m);
      var line = new OrderLine(product, 3);

      // run the real world simulator
      new Thread((ParameterizedThreadStart) RealWorldSimulator) {
        IsBackground = true
      }.Start(line);

      // run this test several times - like the function GetValue() might be called
      // more than once in a real world application
      for (int i = 0; i < 100; i++)
        TestOrderLineValue(line);

    }

    public static void TestOrderLineValue(OrderLine line) {
      try {
        if (line.GetValue( ) == (3 * 16.99m))
          Console.WriteLine("Everything's cool");
        else
          Console.WriteLine("Bugger, wrong value!");
      }
      catch (Exception ex) {
        Console.WriteLine("Caught exception: " + ex.Message);
      }
    }

    static void RealWorldSimulator(object orderLineObj) {
      var orderLine = (OrderLine) orderLineObj;
      var random = new Random( );
      while (true) {
        // do things to the order line, in the way preferred by the API
        orderLine = orderLine.ChangeCount(random.Next(100) + 1);
      }
    }

  }

  public class Product {
    private readonly string name;
    public string Name { get { return name; } }
    private readonly decimal price;
    public decimal Price { get { return price; } }

    public Product(string name, decimal price) {
      this.name = name;
      this.price = price;
    }
  }

  public class OrderLine {
    private readonly Product product;
    public Product Product { get { return product; } }
    private readonly int count;
    public int Count { get { return count; } }

    public OrderLine ChangeCount(int newCount) {
      return new OrderLine(product, newCount);
    }

    public decimal GetValue( ) {
      return Product.Price * Count;
    }

    public OrderLine(Product product, int count) {
      this.product = product;
      this.count = count;
    }
  }
}
