﻿// Copyright 2009-2014 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;

namespace MutableData {
  class Program {
    static void Main(string[] args) {
      var product = new Product { Name = "Rubber boat", Price = 16.99m };
      var line = new OrderLine { Product = product, Count = 3 };

      // run the real world simulator
      new Thread((ParameterizedThreadStart) RealWorldSimulator) {
        IsBackground = true
      }.Start(line);

      // run this test several times - like the function GetValue() might be called
      // more than once in a real world application
      for (int i = 0; i < 100; i++)
        TestOrderLineValue(line);

    }

    static void TestOrderLineValue(OrderLine line) {
      try {
        if (line.GetValue( ) == (3 * 16.99m))
          Console.WriteLine("Everything's cool");
        else
          Console.WriteLine("Bugger, wrong value!");
      }
      catch (Exception ex) {
        Console.WriteLine("Caught exception: " + ex.Message);
      }
    }

    static void RealWorldSimulator(object orderLineObj) {
      var orderLine = (OrderLine) orderLineObj;
      var random = new Random();
      while (true) {
        // do things to the order line, in the way preferred by the API
        orderLine.Count = random.Next(100) + 1;
        
        // orderLine.Product  might change as well, we'll leave it alone
      }
    }
  }

  public class Product {
    public string Name { get; set; }
    public decimal Price { get; set; }
  }

  public class OrderLine {
    public Product Product { get; set; }
    public int Count { get; set; }

    public decimal GetValue( ) {
      return Product.Price * Count;
    }
  }

}
